package dev.fitko.fitconnect.core;

import com.nimbusds.jose.jwk.RSAKey;
import com.nimbusds.jwt.SignedJWT;
import dev.fitko.fitconnect.api.domain.model.destination.Destination;
import dev.fitko.fitconnect.api.domain.model.event.Event;
import dev.fitko.fitconnect.api.domain.model.event.EventLogEntry;
import dev.fitko.fitconnect.api.domain.model.event.EventPayload;
import dev.fitko.fitconnect.api.domain.model.event.authtags.AuthenticationTags;
import dev.fitko.fitconnect.api.domain.model.metadata.Metadata;
import dev.fitko.fitconnect.api.domain.model.metadata.attachment.AttachmentForValidation;
import dev.fitko.fitconnect.api.domain.model.submission.Submission;
import dev.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import dev.fitko.fitconnect.api.domain.validation.ValidationResult;
import dev.fitko.fitconnect.api.exceptions.DecryptionException;
import dev.fitko.fitconnect.api.exceptions.EventCreationException;
import dev.fitko.fitconnect.api.exceptions.RestApiException;
import dev.fitko.fitconnect.api.services.Subscriber;
import dev.fitko.fitconnect.api.services.crypto.CryptoService;
import dev.fitko.fitconnect.api.services.events.EventLogService;
import dev.fitko.fitconnect.api.services.events.SecurityEventService;
import dev.fitko.fitconnect.api.services.submission.SubmissionService;
import dev.fitko.fitconnect.api.services.validation.ValidationService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Set;
import java.util.UUID;

public class SubmissionSubscriber implements Subscriber {

    private static final Logger LOGGER = LoggerFactory.getLogger(SubmissionSubscriber.class);

    private final SubmissionService submissionService;
    private final CryptoService cryptoService;
    private final ValidationService validationService;
    private final EventLogService eventLogService;
    private final SecurityEventService securityEventService;

    public SubmissionSubscriber(final SubmissionService submissionService,
                                final EventLogService eventLogService,
                                final CryptoService cryptoService,
                                final ValidationService validationService,
                                final SecurityEventService securityEventService) {
        this.submissionService = submissionService;
        this.eventLogService = eventLogService;
        this.cryptoService = cryptoService;
        this.validationService = validationService;
        this.securityEventService = securityEventService;
    }

    @Override
    public byte[] decryptStringContent(final RSAKey privateKey, final String encryptedContent) throws DecryptionException {
        LOGGER.info("Decrypting string");
        return cryptoService.decryptToBytes(privateKey, encryptedContent);
    }

    @Override
    public Set<SubmissionForPickup> pollAvailableSubmissionsForDestination(final UUID destinationId, final int offset, final int limit) {
        LOGGER.info("Loading submission {}-{} for destination {}", offset, limit, destinationId);
        return submissionService.pollAvailableSubmissionsForDestination(destinationId, offset, limit).getSubmissions();
    }

    @Override
    public Set<SubmissionForPickup> pollAvailableSubmissions(final int offset, final int limit) {
        LOGGER.info("Loading submission {}-{}", offset, limit);
        return submissionService.pollAvailableSubmissions(offset, limit).getSubmissions();
    }

    @Override
    public Submission getSubmission(final UUID submissionId) throws RestApiException {
        LOGGER.info("Loading submission {}", submissionId);
        return submissionService.getSubmission(submissionId);
    }

    @Override
    public String fetchAttachment(final UUID submissionId, final UUID attachmentId) throws RestApiException {
        LOGGER.info("Loading attachment {} for submission {}", attachmentId, submissionId);
        return submissionService.getAttachment(submissionId, attachmentId);
    }

    @Override
    public List<EventLogEntry> getEventLog(final UUID caseId, final UUID destinationId) throws RestApiException {
        LOGGER.info("Loading event log for destination {}", destinationId);
        return eventLogService.getEventLog(caseId, destinationId);
    }
    @Override
    public AuthenticationTags getAuthenticationTagsForEvent(final Event event, final Submission submission) {
        LOGGER.info("Loading authentication tags of {} event for submission {}", event, submission.getSubmissionId());
        return eventLogService.getAuthenticationTagsForEvent(event, submission);
    }

    @Override
    public ValidationResult validateMetadata(final Metadata metadata, final Submission submission, final AuthenticationTags authenticationTags) {
        LOGGER.info("Validating metadata");
        final Destination destination = submissionService.getDestination(submission.getDestinationId());
        return validationService.validateMetadata(metadata, submission, destination, authenticationTags);
    }

    @Override
    public ValidationResult validateData(final byte[] data, final Submission submission, final Metadata metadata, final AuthenticationTags authenticationTags) {
        LOGGER.info("Validating data");
        return validationService.validateData(data, submission, metadata, authenticationTags);
    }

    @Override
    public ValidationResult validateAttachments(final List<AttachmentForValidation> attachmentsForValidation, final AuthenticationTags authenticationTags) {
        LOGGER.info("Validating attachments");
        return validationService.validateAttachments(attachmentsForValidation, authenticationTags);
    }

    @Override
    public ValidationResult validateHashIntegrity(final String originalHash, final byte[] data) {
        LOGGER.info("Validating hash integrity");
        return validationService.validateHashIntegrity(originalHash, data);
    }

    @Override
    public ValidationResult validateCallback(final String hmac, final Long timestamp, final String httpBody, final String callbackSecret) {
        LOGGER.info("Validating callback integrity");
        return validationService.validateCallback(hmac, timestamp, httpBody, callbackSecret);
    }

    @Override
    public void acceptSubmission(final EventPayload eventPayload) throws RestApiException, EventCreationException {
        LOGGER.info("Accepting submission");
        final SignedJWT confirmedSubmissionEvent = securityEventService.createAcceptSubmissionEvent(eventPayload);
        eventLogService.sendEvent(eventPayload.getCaseId(), confirmedSubmissionEvent.serialize());
        LOGGER.info("CONFIRMED submission {} successfully", eventPayload.getSubmissionId());
    }

    @Override
    public void rejectSubmission(final EventPayload eventPayload) throws RestApiException, EventCreationException  {
        LOGGER.info("Rejecting submission");
        final SignedJWT rejectSubmissionEvent = securityEventService.createRejectSubmissionEvent(eventPayload);
        eventLogService.sendEvent(eventPayload.getCaseId(), rejectSubmissionEvent.serialize());
        LOGGER.info("REJECTED submission {}", eventPayload.getSubmissionId());
    }
}
